@@ -37,7 +37,7 @@ The simplest assertion would be to assert that the template matches a given
 template.
 
 ```python
-expected = {
+template.template_matches({
     "Resources": {
         "BarLogicalId": {
             "Type": "Foo::Bar",
@@ -46,9 +46,7 @@ expected = {
             }
         }
     }
-}
-
-template.template_matches(expected)
+})
 ```
 
 By default, the `templateMatches()` API will use the an 'object-like' comparison,
@@ -84,23 +82,21 @@ The following code asserts that the `Properties` section of a resource of type
 `Foo::Bar` contains the specified properties -
 
 ```python
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Foo": "Bar",
     "Baz": 5,
     "Qux": ["Waldo", "Fred"]
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 ```
 
 Alternatively, if you would like to assert the entire resource definition, you
 can use the `hasResource()` API.
 
 ```python
-expected = {
+template.has_resource("Foo::Bar", {
     "Properties": {"Foo": "Bar"},
     "DependsOn": ["Waldo", "Fred"]
-}
-template.has_resource("Foo::Bar", expected)
+})
 ```
 
 Beyond assertions, the module provides APIs to retrieve matching resources.
@@ -128,21 +124,17 @@ template.has_output("Foo", expected)
 If you want to match against all Outputs in the template, use `*` as the `logicalId`.
 
 ```python
-expected = {
+template.has_output("*", {
     "Value": "Bar",
     "Export": {"Name": "ExportBaz"}
-}
-template.has_output("*", expected)
+})
 ```
 
 `findOutputs()` will return a set of outputs that match the `logicalId` and `props`,
 and you can use the `'*'` special case as well.
 
 ```python
-expected = {
-    "Value": "Fred"
-}
-result = template.find_outputs("*", expected)
+result = template.find_outputs("*", {"Value": "Fred"})
 expect(result.Foo).to_equal({"Value": "Fred", "Description": "FooFred"})
 expect(result.Bar).to_equal({"Value": "Fred", "Description": "BarFred"})
 ```
@@ -182,20 +174,18 @@ level, the list of keys in the target is a subset of the provided pattern.
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.object_like({
         "Wobble": "Flob"
     })
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.object_like({
         "Brew": "Coffee"
     })
-}
-template.has_resource_properties("Foo::Bar", unexpected)
+})
 ```
 
 The `Match.objectEquals()` API can be used to assert a target as a deep exact
@@ -223,20 +213,18 @@ or outside of any matchers.
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.object_like({
         "Bob": Match.absent()
     })
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.object_like({
         "Wobble": Match.absent()
     })
-}
-template.has_resource_properties("Foo::Bar", unexpected)
+})
 ```
 
 The `Match.anyValue()` matcher can be used to specify that a specific value should be found
@@ -261,20 +249,18 @@ This matcher can be combined with any of the other matchers.
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": {
         "Wobble": [Match.any_value(), "Flip"]
     }
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": {
         "Wimble": Match.any_value()
     }
-}
-template.has_resource_properties("Foo::Bar", unexpected)
+})
 ```
 
 ### Array Matchers
@@ -297,16 +283,14 @@ This API will perform subset match on the target.
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.array_with(["Flob"])
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = Match.object_like({
+template.has_resource_properties("Foo::Bar", Match.object_like({
     "Fred": Match.array_with(["Wobble"])
-})
-template.has_resource_properties("Foo::Bar", unexpected)
+}))
 ```
 
 *Note:* The list of items in the pattern array should be in order as they appear in the
@@ -334,16 +318,14 @@ not match the pattern specified.
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": Match.not(["Flob"])
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = Match.object_like({
+template.has_resource_properties("Foo::Bar", Match.object_like({
     "Fred": Match.not(["Flob", "Cat"])
-})
-template.has_resource_properties("Foo::Bar", unexpected)
+}))
 ```
 
 ### Serialized JSON
@@ -370,20 +352,18 @@ The `Match.serializedJson()` matcher allows deep matching within a stringified J
 # }
 
 # The following will NOT throw an assertion error
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Baz": Match.serialized_json({
         "Fred": Match.array_with(["Waldo"])
     })
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 # The following will throw an assertion error
-unexpected = {
+template.has_resource_properties("Foo::Bar", {
     "Baz": Match.serialized_json({
         "Fred": ["Waldo", "Johnny"]
     })
-}
-template.has_resource_properties("Foo::Bar", unexpected)
+})
 ```
 
 ## Capturing Values
@@ -408,15 +388,78 @@ matching resource.
 
 fred_capture = Capture()
 waldo_capture = Capture()
-expected = {
+template.has_resource_properties("Foo::Bar", {
     "Fred": fred_capture,
     "Waldo": ["Qix", waldo_capture]
-}
-template.has_resource_properties("Foo::Bar", expected)
+})
 
 fred_capture.as_array() # returns ["Flob", "Cat"]
 waldo_capture.as_string()
 ```
+
+With captures, a nested pattern can also be specified, so that only targets
+that match the nested pattern will be captured. This pattern can be literals or
+further Matchers.
+
+```python
+# Given a template -
+# {
+#   "Resources": {
+#     "MyBar1": {
+#       "Type": "Foo::Bar",
+#       "Properties": {
+#         "Fred": ["Flob", "Cat"],
+#       }
+#     }
+#     "MyBar2": {
+#       "Type": "Foo::Bar",
+#       "Properties": {
+#         "Fred": ["Qix", "Qux"],
+#       }
+#     }
+#   }
+# }
+
+capture = Capture(Match.array_with(["Cat"]))
+template.has_resource_properties("Foo::Bar", {
+    "Fred": capture
+})
+
+capture.as_array()
+```
+
+When multiple resources match the given condition, each `Capture` defined in
+the condition will capture all matching values. They can be paged through using
+the `next()` API. The following example illustrates this -
+
+```python
+# Given a template -
+# {
+#   "Resources": {
+#     "MyBar": {
+#       "Type": "Foo::Bar",
+#       "Properties": {
+#         "Fred": "Flob",
+#       }
+#     },
+#     "MyBaz": {
+#       "Type": "Foo::Bar",
+#       "Properties": {
+#         "Fred": "Quib",
+#       }
+#     }
+#   }
+# }
+
+fred_capture = Capture()
+template.has_resource_properties("Foo::Bar", {
+    "Fred": fred_capture
+})
+
+fred_capture.as_string() # returns "Flob"
+fred_capture.next() # returns true
+fred_capture.as_string()
+```
 '''
 import abc
 import builtins
@@ -536,9 +579,149 @@ class _MatchProxy(Match):
 typing.cast(typing.Any, Match).__jsii_proxy_class__ = lambda : _MatchProxy
 
 
+@jsii.data_type(
+    jsii_type="@aws-cdk/assertions.MatchCapture",
+    jsii_struct_bases=[],
+    name_mapping={"capture": "capture", "value": "value"},
+)
+class MatchCapture:
+    def __init__(self, *, capture: "Capture", value: typing.Any) -> None:
+        '''Information about a value captured during match.
+
+        :param capture: The instance of Capture class to which this capture is associated with.
+        :param value: The value that was captured.
+
+        :exampleMetadata: fixture=_generated
+
+        Example::
+
+            # The code below shows an example of how to instantiate this type.
+            # The values are placeholders you should change.
+            import aws_cdk.assertions as assertions
+            
+            # capture is of type Capture
+            # value is of type object
+            
+            match_capture = assertions.MatchCapture(
+                capture=capture,
+                value=value
+            )
+        '''
+        self._values: typing.Dict[str, typing.Any] = {
+            "capture": capture,
+            "value": value,
+        }
+
+    @builtins.property
+    def capture(self) -> "Capture":
+        '''The instance of Capture class to which this capture is associated with.'''
+        result = self._values.get("capture")
+        assert result is not None, "Required property 'capture' is missing"
+        return typing.cast("Capture", result)
+
+    @builtins.property
+    def value(self) -> typing.Any:
+        '''The value that was captured.'''
+        result = self._values.get("value")
+        assert result is not None, "Required property 'value' is missing"
+        return typing.cast(typing.Any, result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "MatchCapture(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
+
+@jsii.data_type(
+    jsii_type="@aws-cdk/assertions.MatchFailure",
+    jsii_struct_bases=[],
+    name_mapping={"matcher": "matcher", "message": "message", "path": "path"},
+)
+class MatchFailure:
+    def __init__(
+        self,
+        *,
+        matcher: "Matcher",
+        message: builtins.str,
+        path: typing.Sequence[builtins.str],
+    ) -> None:
+        '''Match failure details.
+
+        :param matcher: The matcher that had the failure.
+        :param message: Failure message.
+        :param path: The relative path in the target where the failure occurred. If the failure occurred at root of the match tree, set the path to an empty list. If it occurs in the 5th index of an array nested within the 'foo' key of an object, set the path as ``['/foo', '[5]']``.
+
+        :exampleMetadata: fixture=_generated
+
+        Example::
+
+            # The code below shows an example of how to instantiate this type.
+            # The values are placeholders you should change.
+            import aws_cdk.assertions as assertions
+            
+            # matcher is of type Matcher
+            
+            match_failure = assertions.MatchFailure(
+                matcher=matcher,
+                message="message",
+                path=["path"]
+            )
+        '''
+        self._values: typing.Dict[str, typing.Any] = {
+            "matcher": matcher,
+            "message": message,
+            "path": path,
+        }
+
+    @builtins.property
+    def matcher(self) -> "Matcher":
+        '''The matcher that had the failure.'''
+        result = self._values.get("matcher")
+        assert result is not None, "Required property 'matcher' is missing"
+        return typing.cast("Matcher", result)
+
+    @builtins.property
+    def message(self) -> builtins.str:
+        '''Failure message.'''
+        result = self._values.get("message")
+        assert result is not None, "Required property 'message' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def path(self) -> typing.List[builtins.str]:
+        '''The relative path in the target where the failure occurred.
+
+        If the failure occurred at root of the match tree, set the path to an empty list.
+        If it occurs in the 5th index of an array nested within the 'foo' key of an object,
+        set the path as ``['/foo', '[5]']``.
+        '''
+        result = self._values.get("path")
+        assert result is not None, "Required property 'path' is missing"
+        return typing.cast(typing.List[builtins.str], result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "MatchFailure(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
+
 class MatchResult(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/assertions.MatchResult"):
     '''The result of ``Match.test()``.
 
+    :exampleMetadata: fixture=_generated
+
     Example::
 
         # The code below shows an example of how to instantiate this type.
@@ -565,6 +748,14 @@ class MatchResult(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/assertions.MatchR
         '''
         return typing.cast("MatchResult", jsii.invoke(self, "compose", [id, inner]))
 
+    @jsii.member(jsii_name="finished")
+    def finished(self) -> "MatchResult":
+        '''Prepare the result to be analyzed.
+
+        This API *must* be called prior to analyzing these results.
+        '''
+        return typing.cast("MatchResult", jsii.invoke(self, "finished", []))
+
     @jsii.member(jsii_name="hasFailed")
     def has_failed(self) -> builtins.bool:
         '''Does the result contain any failures.
@@ -580,18 +771,47 @@ class MatchResult(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/assertions.MatchR
         path: typing.Sequence[builtins.str],
         message: builtins.str,
     ) -> "MatchResult":
-        '''Push a new failure into this result at a specific path.
-
-        If the failure occurred at root of the match tree, set the path to an empty list.
-        If it occurs in the 5th index of an array nested within the 'foo' key of an object,
-        set the path as ``['/foo', '[5]']``.
+        '''(deprecated) DEPRECATED.
 
         :param matcher: -
-        :param path: the path at which the failure occurred.
-        :param message: the failure.
+        :param path: -
+        :param message: -
+
+        :deprecated: use recordFailure()
+
+        :stability: deprecated
         '''
         return typing.cast("MatchResult", jsii.invoke(self, "push", [matcher, path, message]))
 
+    @jsii.member(jsii_name="recordCapture")
+    def record_capture(self, *, capture: "Capture", value: typing.Any) -> None:
+        '''Record a capture against in this match result.
+
+        :param capture: The instance of Capture class to which this capture is associated with.
+        :param value: The value that was captured.
+        '''
+        options = MatchCapture(capture=capture, value=value)
+
+        return typing.cast(None, jsii.invoke(self, "recordCapture", [options]))
+
+    @jsii.member(jsii_name="recordFailure")
+    def record_failure(
+        self,
+        *,
+        matcher: "Matcher",
+        message: builtins.str,
+        path: typing.Sequence[builtins.str],
+    ) -> "MatchResult":
+        '''Record a new failure into this result at a specific path.
+
+        :param matcher: The matcher that had the failure.
+        :param message: Failure message.
+        :param path: The relative path in the target where the failure occurred. If the failure occurred at root of the match tree, set the path to an empty list. If it occurs in the 5th index of an array nested within the 'foo' key of an object, set the path as ``['/foo', '[5]']``.
+        '''
+        failure = MatchFailure(matcher=matcher, message=message, path=path)
+
+        return typing.cast("MatchResult", jsii.invoke(self, "recordFailure", [failure]))
+
     @jsii.member(jsii_name="toHumanStrings")
     def to_human_strings(self) -> typing.List[builtins.str]:
         '''Get the list of failures as human readable strings.'''
@@ -631,20 +851,18 @@ class Matcher(
         # }
         
         # The following will NOT throw an assertion error
-        expected = {
+        template.has_resource_properties("Foo::Bar", {
             "Baz": Match.serialized_json({
                 "Fred": Match.array_with(["Waldo"])
             })
-        }
-        template.has_resource_properties("Foo::Bar", expected)
+        })
         
         # The following will throw an assertion error
-        unexpected = {
+        template.has_resource_properties("Foo::Bar", {
             "Baz": Match.serialized_json({
                 "Fred": ["Waldo", "Johnny"]
             })
-        }
-        template.has_resource_properties("Foo::Bar", unexpected)
+        })
     '''
 
     def __init__(self) -> None:
@@ -717,6 +935,8 @@ class Template(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/assertions.Template"
     Typically used, as part of unit tests, to validate that the rendered
     CloudFormation template has expected resources and properties.
 
+    :exampleMetadata: nofixture
+
     Example::
 
         from aws_cdk.core import Stack
@@ -885,27 +1105,34 @@ class Capture(
         #     "MyBar": {
         #       "Type": "Foo::Bar",
         #       "Properties": {
-        #         "Fred": ["Flob", "Cat"],
-        #         "Waldo": ["Qix", "Qux"],
+        #         "Fred": "Flob",
+        #       }
+        #     },
+        #     "MyBaz": {
+        #       "Type": "Foo::Bar",
+        #       "Properties": {
+        #         "Fred": "Quib",
         #       }
         #     }
         #   }
         # }
         
         fred_capture = Capture()
-        waldo_capture = Capture()
-        expected = {
-            "Fred": fred_capture,
-            "Waldo": ["Qix", waldo_capture]
-        }
-        template.has_resource_properties("Foo::Bar", expected)
+        template.has_resource_properties("Foo::Bar", {
+            "Fred": fred_capture
+        })
         
-        fred_capture.as_array() # returns ["Flob", "Cat"]
-        waldo_capture.as_string()
+        fred_capture.as_string() # returns "Flob"
+        fred_capture.next() # returns true
+        fred_capture.as_string()
     '''
 
-    def __init__(self) -> None:
-        jsii.create(self.__class__, self, [])
+    def __init__(self, pattern: typing.Any = None) -> None:
+        '''Initialize a new capture.
+
+        :param pattern: a nested pattern or Matcher. If a nested pattern is provided ``objectLike()`` matching is applied.
+        '''
+        jsii.create(self.__class__, self, [pattern])
 
     @jsii.member(jsii_name="asArray")
     def as_array(self) -> typing.List[typing.Any]:
@@ -947,6 +1174,14 @@ class Capture(
         '''
         return typing.cast(builtins.str, jsii.invoke(self, "asString", []))
 
+    @jsii.member(jsii_name="next")
+    def next(self) -> builtins.bool:
+        '''When multiple results are captured, move the iterator to the next result.
+
+        :return: true if another capture is present, false otherwise
+        '''
+        return typing.cast(builtins.bool, jsii.invoke(self, "next", []))
+
     @jsii.member(jsii_name="test")
     def test(self, actual: typing.Any) -> MatchResult:
         '''Test whether a target matches the provided pattern.
@@ -971,6 +1206,8 @@ class Capture(
 __all__ = [
     "Capture",
     "Match",
+    "MatchCapture",
+    "MatchFailure",
     "MatchResult",
     "Matcher",
     "Template",
